<?php
/*

  Copyright (C) 2005-2011 UserDot
  Visit XennoBB at www.userdot.net/projects/xennobb
  Originally based on PunBB www.punbb.org

  XennoBB is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published
  by the Free Software Foundation; either version 2 of the License,
  or (at your option) any later version.

  XennoBB is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
  MA 02110-1301 USA

 */
if (!defined('IN_FORUM'))
    exit('No direct script access allowed');

class BB_Settings
{

    var $database;
    var $table;
    var $namespace;
    var $settings = array();

    function BB_Settings(&$database, $table)
    {
        $this->database = $database;
        $this->table = $table;
        $this->namespace = '';
        $this->__Get_Settings();
    }

    function Export()
    {
        foreach ($this->settings as $key => $value)
        {
            if (!defined($key))
                define($key, $value['value']);
        }
    }

    function Set_Namespace($namespace)
    {
        if (preg_match('/^[a-z_]+$/', $namespace))
            $this->namespace = $namespace;
    }

    function Save($setting_id, $value, $type = null, $label = null)
    {
        if ($this->namespace == '')
            return false;
        if ($type == 'double')
            $type = 'float';
        elseif ($type === null)
        {
            if ($this->__Is_Setting($setting_id))
                $type = gettype($this->settings[$setting_id]['value']);
            else
                $type = 'string';
        }
        elseif ($type != 'boolean' && $type != 'integer' && $type != 'float')
            $type = 'string';
        if ($label == null && $this->__Is_Setting($setting_id))
            $label = $this->settings[$setting_id]['label'];
        if (!preg_match('/^bb_[a-zA-Z0-9_]+$/', $setting_id))
            return false;
        settype($value, $type);
        if ($this->__Is_Setting($setting_id) && $this->namespace == $this->settings[$setting_id]['namespace'])
            $sql = "UPDATE " . $this->table . " SET value = '" . $this->database->escape($value) . "', type = '" . $this->database->escape($type) . "', label = '" . $this->database->escape($label) . "' WHERE id = '" . $this->database->escape($setting_id) . "' ";
        else
            $sql = 'INSERT INTO ' . $this->table . ' (id, namespace, value, type, label) VALUES (' . "'" . $this->database->escape($setting_id) . "', '" . $this->database->escape($this->namespace) . "', '" . $this->database->escape($value) . "', '" . $this->database->escape($type) . "', '" . $this->database->escape($label) . "'" . ') ';
        if ($this->database->query($sql))
            return true;
        else
            BB_Functions::Error('Cannot update XennoBB configuration.', __FILE__, __LINE__, $this->database->error());
    }

    function Remove($setting_id)
    {
        if ($this->database->query('DELETE FROM ' . $this->table . ' ' . "WHERE id = '" . $this->database->escape($setting_id) . "' "))
            return true;
        else
            BB_Functions::Error('Cannot update XennoBB configuration.', __FILE__, __LINE__, $this->database->error());
    }

    function __Get_Settings()
    {
        $result = $this->database->query('SELECT id, namespace, value, type, label FROM ' . $this->table . ' ORDER BY namespace, id DESC ') or BB_Functions::Error('Cannot get XennoBB configuration from database.', __FILE__, __LINE__, $this->database->error());
        $cur_setting = array();
        while ($cur_setting = $this->database->fetch_assoc($result))
        {
            $id = BB_Input::Trim($cur_setting['id']);
            $value = $cur_setting['value'];
            $type = $cur_setting['type'];
            if ($type == 'float' || $type == 'double')
                $type = (version_compare(PHP_VERSION, '4.2.0', 'lt')) ? 'double' : 'float';
            elseif ($type != 'boolean' && $type != 'integer')
                $type = 'string';
            settype($value, $type);
            $this->settings[$id] = array(
                'namespace' => $cur_setting['namespace'],
                'value' => $value,
                'label' => (string) $cur_setting['label']
            );
        }
        return false;
    }

    function __Is_Setting($setting_id)
    {
        return isset($this->settings[$setting_id]);
    }

}

?>
