<?php
/*

  Copyright (C) 2005-2011 UserDot
  Visit XennoBB at www.userdot.net/projects/xennobb
  Originally based on PunBB www.punbb.org

  XennoBB is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published
  by the Free Software Foundation; either version 2 of the License,
  or (at your option) any later version.

  XennoBB is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
  MA 02110-1301 USA

 */
if (!defined('IN_FORUM'))
    exit('No direct script access allowed');

class BB_Files
{

    function Get_Directory_List($name)
    {
        static $filelist = array();
        static $dirlist = array();
        $exclude_list = array('.', '..', '.svn');
        if (empty($res))
            $res = array();
        $name = preg_replace('|/$|', '', $name);
        if (!is_dir($name))
            return false;
        $dirlist[] = $name;
        $d = dir($name);
        while ($entry = $d->read())
        {
            if (!in_array($entry, $exclude_list))
            {
                if (is_dir($name . '/' . $entry))
                {
                    if ($entry != 'CVS')
                        BB_Files::Get_Directory_List($name . '/' . $entry);
                }
                else
                    $filelist[] = $name . '/' . $entry;
            }
        }
        $d->close();
        return array('dirs' => $dirlist, 'files' => $filelist);
    }

    function Make_Package($name, $dir, $remove_path = '', $gzip = true)
    {
        if ($gzip && !function_exists('gzcompress'))
            return false;
        if (($filelist = BB_Files::Get_Directory_List($dir)) === false)
            return false;
        $res = array('name' => $name, 'dirs' => array(), 'files' => array());
        foreach ($filelist['dirs'] as $v)
            $res['dirs'][] = preg_replace('/^' . preg_quote($remove_path, '/') . '/', '', $v);
        foreach ($filelist['files'] as $v)
        {
            $f_content = base64_encode(file_get_contents($v));
            $v = preg_replace('/^' . preg_quote($remove_path, '/') . '/', '', $v);
            $res['files'][$v] = $f_content;
        }
        $res = serialize($res);
        if ($gzip)
            $res = gzencode($res);
        return $res;
    }

    function Scan_Directory($name, $order = 0)
    {
        $res = array();
        if (($dh = @opendir($name)) !== false)
        {
            while (($f = readdir($dh)) !== false)
                $res[] = $f;
            closedir($dh);
            sort($res);
            if ($order == 1)
                rsort($res);
            return $res;
        }
        else
            return false;
    }

    function Is_Deletable($name)
    {
        if (is_file($name))
            return is_writable(dirname($name));
        elseif (is_dir($name))
            return (is_writable(dirname($name)) && count(BB_Files::Scan_Directory($name)) <= 2);
    }

    function Secure_File($name)
    {
        if (is_file($name))
        {
            @chmod($name, 0600);
            if (is_readable($name))
                return true;
            else
            {
                @chmod($name, 0660);
                if (is_readable($name))
                    return true;
                else
                    @chmod($name, 0666);
            }
        }
    }

    function Create_Directory($name)
    {
        if (@mkdir($name, fileperms(dirname($name))) === false)
            return false;
        @chmod($name, fileperms(dirname($name)));
    }

    function Delete_Tree($name)
    {
        $current_dir = opendir($name);
        while ($entryname = readdir($current_dir))
        {
            if (is_dir($name . '/' . $entryname) and ($entryname != '.' and $entryname != '..'))
            {
                if (!BB_Files::Delete_Tree($name . '/' . $entryname))
                    return false;
            }
            elseif ($entryname != '.' and $entryname != '..')
            {
                if (!@unlink($name . '/' . $entryname))
                    return false;
            }
        }
        closedir($current_dir);
        return @rmdir($name);
    }

    function Touch($name)
    {
        if (is_writable($name))
        {
            $c = implode('', file($name));
            if ($fp = @fopen($name, 'w'))
            {
                fwrite($fp, $c, strlen($c));
                fclose($fp);
            }
        }
    }

    function Size($size)
    {
        $kb = 1024;
        $mb = 1024 * $kb;
        $gb = 1024 * $mb;
        $tb = 1024 * $gb;
        if ($size < $kb)
            return $size . " B";
        else if ($size < $mb)
            return round($size / $kb, 2) . " KB";
        else if ($size < $gb)
            return round($size / $mb, 2) . " MB";
        else if ($size < $tb)
            return round($size / $gb, 2) . " GB";
        else
            return round($size / $tb, 2) . " TB";
    }

    function Put_Content($name, $file_content)
    {
        if (is_writable($name))
        {
            if ($fp = @fopen($name, 'w'))
            {
                fwrite($fp, $file_content, strlen($file_content));
                fclose($fp);
                return true;
            }
        }
        return false;
    }

    function Copy_Remote($src, $dest)
    {
        if (($fp1 = @fopen($src, 'r')) === false)
            return 'An error occured while downloading the file.';
        else
        {
            if (($fp2 = @fopen($dest, 'w')) === false)
            {
                fclose($fp1);
                return 'An error occured while writing the file.';
            }
            else
            {
                while (($buffer = fgetc($fp1)) !== false)
                    fwrite($fp2, $buffer);
                fclose($fp1);
                fclose($fp2);
                return true;
            }
        }
    }

}

class BB_Path
{

    function Real($p, $strict = true)
    {
        $os = (DIRECTORY_SEPARATOR == '\\') ? 'win' : 'nix';
        if ($os == 'win')
            $_abs = preg_match('/^\w+:/', $p);
        else
            $_abs = substr($p, 0, 1) == '/';
        if ($os == 'win')
            $p = str_replace('\\', '/', $p);
        if (!$_abs)
            $p = dirname($_SERVER['SCRIPT_FILENAME']) . '/' . $p;
        $p = preg_replace('|/+|', '/', $p);
        if (strlen($p) > 1)
            $p = preg_replace('|/$|', '', $p);
        $_start = '';
        if ($os == 'win')
        {
            list($_start, $p) = explode(':', $p);
            $_start .= ':/';
        }
        else
            $_start = '/';
        $p = substr($p, 1);
        $P = explode('/', $p);
        $res = array();
        for ($i = 0; $i < count($P); $i++)
        {
            if ($P[$i] == '.')
                continue;
            if ($P[$i] == '..')
            {
                if (count($res) > 0)
                    array_pop($res);
            }
            else
                array_push($res, $P[$i]);
        }
        $p = $_start . implode('/', $res);
        if ($strict && !@file_exists($p))
            return false;
        return $p;
    }

}

class BB_File_Manager
{

    var $root;
    var $exclude_list = array();
    var $_types = array('img' => '', 'txt' => '');

    function BB_File_Manager($root_path, $base_path)
    {
        $this->root = BB_Path::Real($root_path);
        $this->base_path = $this->__Clean_Path('/' . $base_path);
        $this->_types['img'] = '.gif|.jpg|.jpeg|.png|.bmp';
        $this->_types['txt'] = '.htm|.html|.php|.php3|.inc|.xml|.txt|.xhtml|.css';
    }

    function Add_Exclusion($name)
    {
        if (is_array($name))
        {
            foreach ($name as $v)
            {
                if (($V = BB_Path::Real($v)) !== false)
                    $this->exclude_list[] = $V;
            }
        }
        elseif (($F = BB_Path::Real($name)) !== false)
            $this->exclude_list[] = $F;
    }

    function Is_Excluded()
    {
        foreach ($this->exclude_list as $v)
        {
            if (strpos($this->root . $this->base_path, $v) === 0)
                return true;
        }
        return false;
    }

    function Get_Directory()
    {
        $dir = $this->__Clean_Path($this->root . $this->base_path);
        if ($dh = @opendir($dir))
        {
            $d_res = $f_res = array();
            while (($file = readdir($dh)) !== false)
            {
                $fname = $dir . '/' . $file;
                $jailed = $this->__Is_Jailed($fname);
                $tmp = array();
                $stat = @stat($fname);
                $tmp['fname'] = $fname;
                $tmp['jail'] = $jailed;
                $tmp['type'] = NULL;
                $tmp['type'] = @filetype($fname);
                $tmp['mtime'] = $stat[9];
                $tmp['size'] = $stat[7];
                $tmp['mode'] = $stat[2];
                $tmp['uid'] = $stat[4];
                $tmp['gid'] = $stat[5];
                $tmp['w'] = $tmp['r'] = $tmp['x'] = $tmp['f'] = $tmp['del'] = false;
                $tmp['d'] = ($file == '..');
                $tmp['l'] = NULL;
                if ($jailed && !in_array($fname, $this->exclude_list))
                {
                    $tmp['w'] = @is_writable($fname);
                    $tmp['r'] = @is_readable($fname);
                    $tmp['x'] = @file_exists($fname . '/.');
                    $tmp['f'] = @is_file($fname);
                    $tmp['d'] = @is_dir($fname);
                    $tmp['l'] = $this->__Get_Relative_Path($fname);
                    $tmp['del'] = ($file != '.') ? BB_Files::Is_Deletable($fname) : false;
                    $tmp['type'] = $this->__Get_Type($fname);
                }
                if (@is_dir($fname))
                    $d_res[$file] = $tmp;
                else
                    $f_res[$file] = $tmp;
            }
            closedir($dh);
            ksort($d_res);
            ksort($f_res);
            return array('dirs' => $d_res, 'files' => $f_res);
        }
        return false;
    }

    function Get_Content()
    {
        if ($this->Is_File() && $this->Is_Readable())
            return file_get_contents($this->root . $this->base_path);
        return NULL;
    }

    function Is_File()
    {
        return is_file($this->root . $this->base_path);
    }

    function Is_Directory()
    {
        return is_dir($this->root . $this->base_path);
    }

    function Is_Writable()
    {
        return is_writable($this->root . $this->base_path);
    }

    function Is_Readable()
    {
        return is_readable($this->root . $this->base_path);
    }

    function Is_Deletable()
    {
        return BB_Files::Is_Deletable($this->root . $this->base_path);
    }

    function Is_Parent_Writable()
    {
        return is_writable(dirname($this->root . $this->base_path));
    }

    function Is_Image()
    {
        return ($this->__Get_Type($this->root . $this->base_path) == 'img');
    }

    function Is_Text()
    {
        return ($this->__Get_Type($this->root . $this->base_path) == 'txt');
    }

    function New_Directory($name)
    {
        $name = str_replace('/', '', $name);
        return BB_Files::Create_Directory($this->root . $this->base_path . '/' . $name);
    }

    function Put_Content($c)
    {
        return BB_Files::Put_Content($this->root . $this->base_path, $c);
    }

    function Rename($name)
    {
        $name = str_replace('/', '', $name);
        $d = $this->root . $this->base_path;
        $n = dirname($d) . '/' . $name;
        return @rename($d, $n);
    }

    function Delete()
    {
        $f = $this->root . $this->base_path;
        if (is_file($f))
        {
            if (@unlink($f) === true)
                return true;
        }
        elseif (is_dir($f))
        {
            if (@rmdir($f) === true)
                return true;
        }
        return false;
    }

    function __Clean_Path($name)
    {
        $name = str_replace('..', '', $name);
        $name = preg_replace('|/{2,}|', '/', $name);
        $name = preg_replace('|/$|', '', $name);
        return $name;
    }

    function __Is_Jailed($name)
    {
        if (($name = BB_Path::Real($name)) !== false)
        {
            if (preg_match('|^' . preg_quote($this->root, '|') . '|', $name))
                return true;
            else
                return false;
        }
        else
            return false;
    }

    function __Get_Relative_Path($name)
    {
        if ($this->__Is_Jailed($name))
        {
            $name = BB_Path::Real($name);
            return preg_replace('|^' . preg_quote($this->root, '|') . '|', '', $f);
        }
        else
            return false;
    }

    function __Get_Type($name)
    {
        global $fm_cf_types;
        if (is_file($name))
        {
            if (strpos(basename($name), '.') === false)
                return 'txt';
            elseif (preg_match('/(' . $this->_types['img'] . ')$/i', basename($name)))
                return 'img';
            elseif (preg_match('/(' . $this->_types['txt'] . ')$/i', basename($name)))
                return 'txt';
        }
        else
            return NULL;
    }

    function Get_File_Extension($name)
    {
        $filename = basename($name);
        $ext = substr($name, strrpos($name, '.') + 1);
        return $ext;
    }

    function Get_Dir_Contents($name, $remove_thumbnails = false)
    {
        $contents = array();
        if (!is_dir($name))
            return $contents;
        $dh = opendir($name);
        while (false !== ($file = readdir($dh)))
        {
            if ($file == '.' || $file == '..' || ($remove_thumbnails && $file == 'thumbnails'))
                continue;
            $contents[] = $file;
        }
        closedir($dh);
        sort($contents);
        return $contents;
    }

    function Get_Dir_File_Count($name)
    {
        return count(BB_File_Manager::Get_Dir_Contents($name));
    }

    function Remove_Directory_Recursive($name)
    {
        $name = realpath($name) . '/';
        if (!is_dir($name))
            return false;
        $files = BB_File_Manager::Get_Dir_Contents($name);
        foreach ($files as $file)
        {
            $file = $name . $file;
            if (is_file($file))
            {
                if (!@unlink($file))
                    return false;
            }
            else
            {
                if (!BB_File_Manager::Remove_Directory_Recursive($file))
                    return false;
            }
        }
        if (!@rmdir($name))
            return false;
        return true;
    }

}

?>
