<?php
/*

  Copyright (C) 2005-2011 UserDot
  Visit XennoBB at www.userdot.net/projects/xennobb
  Originally based on PunBB www.punbb.org

  XennoBB is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published
  by the Free Software Foundation; either version 2 of the License,
  or (at your option) any later version.

  XennoBB is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
  MA 02110-1301 USA

 */
if (!defined('IN_FORUM'))
    exit('No direct script access allowed');

class BB_Image_Upload
{

    function Check_Configuration()
    {
        return true;
    }

    function Get_Mime_Type($filename)
    {
        if (!is_file($filename))
            return '';
        list($width, $height, $type, $attr) = getimagesize($filename);
        return image_type_to_mime_type($type);
    }

    function Show_Problems()
    {
        $log = array();
        if (!BB_Image_Upload::Check_Configuration())
        {
            $log[] = "Image uploading is not configured correctly!";
            return $log;
        }
        $root = bb_upload_path;
        $pids = BB_File_Manager::Get_Dir_Contents($root);
        foreach ($pids as $pid)
        {
            $cur_dir = $root . $pid . '/';
            $images = BB_File_Manager::Get_Dir_Contents($cur_dir, true);
            $icount = count($images);
            if (!is_dir($cur_dir . 'thumbnails'))
            {
                $log[] = "Post $pid: Error in directory structure";
                continue;
            }
            if (!is_writable($cur_dir))
            {
                $log[] = "Post $pid: Directory isn't writable";
                continue;
            }
            if (!is_writable($cur_dir . 'thumbnails'))
            {
                $log[] = "Post $pid: Thumbnails directory isn't writable";
                continue;
            }
            $thumbnails = BB_File_Manager::Get_Dir_Contents($cur_dir . 'thumbnails');
            $tcount = count($thumbnails);
            if ($icount == 0)
            {
                $log[] = "Post $pid: No attached images";
                continue;
            }
            if ($tcount == 0)
            {
                $log[] = "Post $pid: Missing thumbnails";
                continue;
            }
            if ($icount != $tcount)
            {
                $log[] = "Post $pid: Image count doesn't match thumbnail count";
                continue;
            }
            $idx = 0;
            $error = false;
            for ($idx = 0; $idx < $icount; $idx++)
            {
                if ($images[$idx] != $thumbnails[$idx])
                {
                    $error = true;
                    break;
                }
            }
            if ($error)
            {
                $log[] = "Post $pid: Error in filename synchronization";
                continue;
            }
        }
        if (count($log) == 0)
            $log[] = "No problems Found!";
        return $log;
    }

    function Delete_Orphan_Images()
    {
        global $db;
        $log = array();
        if (!BB_Image_Upload::Check_Configuration())
        {
            $log[] = "Image uploading is not configured correctly!";
            return $log;
        }
        $root = bb_upload_path;
        $pids = BB_File_Manager::Get_Dir_Contents($root);
        $sql = 'SELECT p.id FROM ' . $db->prefix . 'posts AS p WHERE (';
        $first = true;
        foreach ($pids as $pid)
        {
            if ($first)
                $first = false;
            else
                $sql .= ' or ';
            $sql .= 'p.id=' . $pid;
        }
        if ($first)
        {
            $log[] = "No problems Found!";
            return $log;
        }
        $sql .= ')';
        $result = $db->query($sql) or BB_Functions::Error($lang['error_998'], __FILE__, __LINE__, $db->error());
        while ($pid = $db->fetch_assoc($result))
        {
            $idx = array_search($pid, $pids);
            array_splice($pids, $idx, 1);
        }
        foreach ($pids as $pid)
        {
            if (BB_File_Manager::Remove_Directory_Recursive($root . $pid . '/'))
                $log[] = "Post $pid: No post with this id - Deleted";
            else
                $log[] = "Post $pid: No post with this id - Unable to Delete";
        }
        if (count($log) == 0)
            $log[] = "No problems Found!";
        return $log;
    }

    function Create_Thumbnail($name, $filename)
    {
        $system = explode('.', $name);
        if (preg_match('/jpg|jpeg/', $system[1]))
            $src_img = imagecreatefromjpeg($name);
        if (preg_match('/png/', $system[1]))
            $src_img = imagecreatefrompng($name);
        if (preg_match('/gif/', $system[1]))
            $src_img = imagecreatefromgif($name);
        $old_x = imageSX($src_img);
        $old_y = imageSY($src_img);
        if ($old_x > $old_y)
        {
            $thumb_w = bb_thumb_width;
            $thumb_h = $old_y * (bb_thumb_height / $old_x);
        }
        if ($old_x < $old_y)
        {
            $thumb_w = $old_x * (bb_thumb_width / $old_y);
            $thumb_h = bb_thumb_height;
        }
        if ($old_x == $old_y)
        {
            $thumb_w = bb_thumb_width;
            $thumb_h = bb_thumb_height;
        }
        $dst_img = ImageCreateTrueColor($thumb_w, $thumb_h);
        imagecopyresampled($dst_img, $src_img, 0, 0, 0, 0, $thumb_w, $thumb_h, $old_x, $old_y);
        if (preg_match("/png/", $system[1]))
            imagepng($dst_img, $filename);
        else if (preg_match("/jpg|jpeg/", $system[1]))
            imagejpeg($dst_img, $filename);
        else
            imagegif($dst_img, $filename);
        imagedestroy($dst_img);
        imagedestroy($src_img);
    }

    function Process_Uploaded_Images($pid)
    {
        global $lang;
        if (!isset($_FILES['image_upload']['error']))
            return '';
        if (!BB_Image_Upload::Check_Configuration())
            return '';
        $result = $lang['Upload Results'] . ":<br />\n";
        $total_uploaded = 0;
        $dest = bb_upload_path . $pid . '/';
        $allowed_ext = bb_allowed_ext;
        $allowed_ext = explode(',', $allowed_ext);
        foreach ($_FILES['image_upload']['error'] as $key => $error)
        {
            if ($error == UPLOAD_ERR_OK)
            {
                $tmp_name = $_FILES["image_upload"]["tmp_name"][$key];
                $name = $_FILES["image_upload"]["name"][$key];
                $file_ext = BB_File_Manager::Get_File_Extension($name);
                if (!in_array(strtolower($file_ext), $allowed_ext) || $file_ext == '')
                {
                    $result .= "$name " . $lang['Extension Banned'] . ".<br />\n";
                    continue;
                }
                if (filesize($tmp_name) > bb_max_size)
                {
                    $result .= "$name " . $lang['Size Too Big'] . ".<br />\n";
                    continue;
                }
                list($width, $height, $type, $attr) = getimagesize($tmp_name);
                if ($width > bb_max_width || $height > bb_max_height)
                {
                    $result .= "$name " . $lang['Dim Too Big'] . ".<br />\n";
                    continue;
                }
                if (!is_dir($dest))
                {
                    mkdir($dest);
                    chmod($dest, 0777);
                    mkdir($dest . 'thumbnails');
                    chmod($dest . 'thumbnails', 0777);
                }
                move_uploaded_file($tmp_name, $dest . $name);
                chmod($dest . $name, 0666);
                BB_Image_Upload::Create_Thumbnail($dest . $name, $dest . 'thumbnails/' . $name);
                $total_uploaded++;
            }
        }
        $result .= $lang['Uploaded'] . ' ' . $total_uploaded . ' ' . $lang['Images'] . "<br /><br />\n";
        return $result;
    }

    function Process_Deleted_Images($pid)
    {
        if (!isset($_POST['delete_image']))
            return;
        if (!BB_Image_Upload::Check_Configuration())
            return;
        $target_dir = bb_upload_path . $pid . '/';
        foreach ($_POST['delete_image'] as $image)
        {
            $image = basename($image);
            if (!is_file($target_dir . $image) || !is_file($target_dir . 'thumbnails/' . $image))
                continue;
            unlink($target_dir . $image);
            unlink($target_dir . 'thumbnails/' . $image);
        }
        if (count(BB_File_Manager::Get_Dir_Contents($target_dir . 'thumbnails/')) == 0)
        {
            rmdir($target_dir . 'thumbnails/');
            rmdir($target_dir);
        }
    }

    function Delete_Images($pid)
    {
        if (!BB_Image_Upload::Check_Configuration())
            return;
        $target_dir = bb_upload_path . $pid . '/';
        if (!is_dir($target_dir))
            return;
        $images = BB_File_Manager::Get_Dir_Contents($target_dir . 'thumbnails/');
        foreach ($images as $image)
            unlink($target_dir . 'thumbnails/' . $image);
        rmdir($target_dir . 'thumbnails/');
        $images = BB_File_Manager::Get_Dir_Contents($target_dir);
        foreach ($images as $image)
            unlink($target_dir . $image);
        rmdir($target_dir);
    }

    function Show_Images($pid, $edit = false)
    {
        global $lang;
        if (!BB_Image_Upload::Check_Configuration())
            return;
        if (!is_dir(bb_upload_path . $pid))
            return;
        $images = BB_File_Manager::Get_Dir_Contents(bb_upload_path . $pid . '/thumbnails/');
        $image_count = count($images);
        $col = 0;
        $idx = 0;
        $column_count = bb_table_cols;
        $column_width = floor(100 / $column_count);
        $image_height = bb_thumb_height;
        $row_height = $image_height + 6;
        if ($edit)
            $row_height += 20;
        $output[] = '';
        if (!$edit)
            $output[] = "\t\t\t\t\t<br />";
        $output[] = "\t\t\t\t\t<div class='image_thumbnails'>";
        $output[] = "\t\t\t\t\t\t";
        $output[] = "\t\t\t\t\t\t\t<hr size=\"1px\" noshade>";
        while ($idx < $image_count)
        {
            $output[] = "\t\t\t\t\t\t\t<div class='image_row' style='height: " . $row_height . "px;'>";
            for ($col = 0; $col < $column_count; $col++, $idx++)
            {
                if ($col == $column_count - 1)
                    $column_width--;
                if ($idx < $image_count)
                {
                    $url_name = urlencode($images[$idx]);
                    $output[] = "\t\t\t\t\t\t\t\t<span class='image_item' style='width: " . $column_width . "%; height: $image_height" . "px;'>";
                    if (!$edit)
                        $output[] = "\t\t\t\t\t\t\t\t\t<a href='gallery.php?" . "pid=$pid&amp;filename=$url_name'>";
                    $output[] = "\t\t\t\t\t\t\t\t\t\t<span class='image_thumbnail' " . "style=\"background-image: url('image.php?" . "pid=$pid&amp;filename=$url_name&amp;preview=true'); " . ($edit ? '' : 'cursor: pointer;') . "\"></span>";
                    if (!$edit)
                        $output[] = "\t\t\t\t\t\t\t\t\t</a>";
                    if ($edit)
                        $output[] = "\t\t\t\t\t\t\t\t\t<label for='delete_image_$idx'>" . "<input type='checkbox' name='delete_image[]' " . "id='delete_image_$idx' value='" . urlencode($images[$idx]) . "' /> Delete</label>";
                    $output[] = "\t\t\t\t\t\t\t\t</span>";
                }
                if ($col == $column_count - 1)
                    $column_width++;
            }
            $output[] = "\t\t\t\t\t\t\t</div>";
        }
        $output[] = "\t\t\t\t\t\t";
        $output[] = "\t\t\t\t\t</div>";
        $output[] = "\t\t\t\t\t<br />\n";
        echo implode("\n", $output);
    }

    function Show($cur_post, $edit = false)
    {
        global $lang, $forum_user;
        if (!BB_Image_Upload::Check_Configuration())
            return;
        $upload_slots = 0;
        if (isset($_POST['upload_slots']))
            $upload_slots = $_POST['upload_slots'];
        if ($cur_post['image_upload'] || $forum_user['g_id'] == USER_ADMIN)
        {
            ?>
            <div class="infldset" style="padding:10px;">
                <div class="rbox">
                    <p><?php echo $lang['image_upload_show']; ?>
                        <select onchange="xBB_Update_Upload_Slots()" size="1" name="upload_slots">
            <?php
            $upload_limit = bb_max_post_images;
            if ($edit)
                $upload_limit -= BB_File_Manager::Get_Dir_File_Count(bb_upload_path . intval($_GET['id']) . '/thumbnails');
            for ($i = 0; $i <= $upload_limit; $i++)
            {
                echo "\t\t\t\t\t\t\t\t<option value='$i' ";
                if ($upload_slots == $i)
                    echo 'selected="selected"';
                echo ">$i</option>\n";
            }
            ?>
                        </select>
            <?php echo $lang['image_upload_slots']; ?></p>
                    <div id="image_upload">
            <?php
            for ($i = 0; $i < $upload_slots; $i++)
                echo "\t\t\t\t\t\t\t\t<p><input type='file' " . "name='image_upload[]' size='70' /></p>\n";
            ?>
                    </div>
                    <p>(<?php echo $lang['image_upload_limits'] . ': ' . bb_max_post_images . ' ' . $lang['image_upload_limits2'] . ' ' . bb_max_width . 'x' . bb_max_height . ' ' . $lang['image_upload_pixels'] . ' ' . round(bb_max_size / 1024); ?> KB)</p>
                </div>
            </div>
            <?php
            if ($edit)
                BB_Image_Upload::Show_Images(intval($_GET['id']), true);
        }
    }

}
?>
