<?php
/*

  Copyright (C) 2005-2011 UserDot
  Visit XennoBB at www.userdot.net/projects/xennobb
  Originally based on PunBB www.punbb.org

  XennoBB is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published
  by the Free Software Foundation; either version 2 of the License,
  or (at your option) any later version.

  XennoBB is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
  MA 02110-1301 USA

 */
if (!defined('IN_FORUM'))
    exit('No direct script access allowed');
@include_once FORUM_ROOT . 'var/cache/cache_smilies.php';
if (!defined('SMILEY_LOADED'))
{
    BB_Cache::Generate_Smiley();
    require FORUM_ROOT . 'var/cache/cache_smilies.php';
}

class BB_Parser
{

    function Preparse_BBCode($text, &$errors, $is_signature = false)
    {
        $a = array('[B]', '[I]', '[U]', '[PHP]', '[/B]', '[/I]', '[/U]', '[/PHP]');
        $b = array('[b]', '[i]', '[u]', '[php]', '[/b]', '[/i]', '[/u]', '[/php]');
        $text = str_replace($a, $b, $text);
        $a = array('#\[url=("|\'|)(.*?)\\1\]\s*#i',
            '#\[url\]\s*#i',
            '#\s*\[/url\]#i',
            '#\[email=("|\'|)(.*?)\\1\]\s*#i',
            '#\[email\]\s*#i',
            '#\s*\[/email\]#i',
            '#\[img\]\s*(.*?)\s*\[/img\]#is',
            '#\[colou?r=("|\'|)(.*?)\\1\](.*?)\[/colou?r\]#is');
        $b = array('[url=$2]',
            '[url]',
            '[/url]',
            '[email=$2]',
            '[email]',
            '[/email]',
            '[img]$1[/img]',
            '[color=$2]$3[/color]');
        if (!$is_signature)
        {
            $a[] = '#\[quote=(&quot;|"|\'|)(.*?)\\1\]\s*#i';
            $a[] = '#\[quote\]\s*#i';
            $a[] = '#\s*\[/quote\]\s*#i';
            $a[] = '#\[code\][\r\n]*(.*?)\s*\[/code\]\s*#is';
            $b[] = '[quote=$1$2$1]';
            $b[] = '[quote]';
            $b[] = '[/quote]' . "\n";
            $b[] = '[code]$1[/code]' . "\n";
        }
        $text = preg_replace($a, $b, $text);
        if (!$is_signature)
        {
            $overflow = BB_Parser::Check_Tag_Order($text, $error);
            if ($error)
                $errors[] = $error;
            else if ($overflow)
                $text = substr($text, 0, $overflow[0]) . substr($text, $overflow[1], (strlen($text) - $overflow[0]));
        }
        else
        {
            global $lang;
            if (preg_match('#\[quote=(&quot;|"|\'|)(.*)\\1\]|\[quote\]|\[/quote\]|\[code\]|\[/code\]#i', $text))
                BB_Functions::Message($lang['Signature quote/code']);
        }
        return trim($text);
    }

    function Check_Tag_Order($text, &$error)
    {
        global $lang;
        $max_depth = 3;
        $cur_index = 0;
        $q_depth = 0;
        while (true)
        {
            if (preg_match('#\[code=?(.*?)\]#s', $text, $matches))
                $c_start = strpos($text, $matches[0]);
            else
                $c_start = 65536;
            $c_end = strpos($text, '[/code]');
            $q_start = strpos($text, '[quote]');
            $q_end = strpos($text, '[/quote]');
            if (preg_match('#\[quote=(&quot;|"|\'|)(.*)\\1\]#sU', $text, $matches))
                $q2_start = strpos($text, $matches[0]);
            else
                $q2_start = 65536;
            if ($c_start === false)
                $c_start = 65536;
            if ($c_end === false)
                $c_end = 65536;
            if ($q_start === false)
                $q_start = 65536;
            if ($q_end === false)
                $q_end = 65536;
            if (min($c_start, $c_end, $q_start, $q_end, $q2_start) == 65536)
                break;
            $q3_start = ($q_start < $q2_start) ? $q_start : $q2_start;
            if ($q3_start < min($q_end, $c_start, $c_end))
            {
                $step = ($q_start < $q2_start) ? 7 : strlen($matches[0]);
                $cur_index += $q3_start + $step;
                if ($q_depth == $max_depth)
                    $overflow_begin = $cur_index - $step;
                ++$q_depth;
                $text = substr($text, $q3_start + $step);
            }
            else if ($q_end < min($q_start, $c_start, $c_end))
            {
                if ($q_depth == 0)
                {
                    $error = $lang['BBCode error'] . ' ' . $lang['BBCode error 1'];
                    return;
                }
                $q_depth--;
                $cur_index += $q_end + 8;
                if ($q_depth == $max_depth)
                    $overflow_end = $cur_index;
                $text = substr($text, $q_end + 8);
            }
            else if ($c_start < min($c_end, $q_start, $q_end))
            {
                $tmp = strpos($text, '[/code]');
                $tmp2 = strpos(substr($text, $c_start + 6), '[code]');
                if ($tmp2 !== false)
                    $tmp2 += $c_start + 6;
                if ($tmp === false || ($tmp2 !== false && $tmp2 < $tmp))
                {
                    $error = $lang['BBCode error'] . ' ' . $lang['BBCode error 2'];
                    return;
                }
                else
                    $text = substr($text, $tmp + 7);
                $cur_index += $tmp + 7;
            }
            else if ($c_end < min($c_start, $q_start, $q_end))
            {
                $error = $lang['BBCode error'] . ' ' . $lang['BBCode error 3'];
                return;
            }
        }
        if ($q_depth)
        {
            $error = $lang['BBCode error'] . ' ' . $lang['BBCode error 4'];
            return;
        }
        else if ($q_depth < 0)
        {
            $error = $lang['BBCode error'] . ' ' . $lang['BBCode error 5'];
            return;
        }
        if (isset($overflow_begin))
            return array($overflow_begin, $overflow_end);
        else
            return null;
    }

    function Split_Text($text, $start, $end)
    {
        $tokens = explode($start, $text);
        $outside[] = $tokens[0];
        $num_tokens = count($tokens);
        for ($i = 1; $i < $num_tokens; ++$i)
        {
            $temp = explode($end, $tokens[$i]);
            $inside[] = $temp[0];
            $outside[] = $temp[1];
        }
        if (bb_indent_num_spaces != 8 && $start == '[code]')
        {
            $spaces = str_repeat(' ', bb_indent_num_spaces);
            $inside = str_replace("\t", $spaces, $inside);
        }
        return array($inside, $outside);
    }

    function Handle_URL_Tag($url, $link = '')
    {
        global $forum_user;
        $full_url = str_replace(array(' ', '\'', '`', '"'), array('%20', '', '', ''), $url);
        if (strpos($url, 'www.') === 0)
            $full_url = 'http://' . $full_url;
        else if (strpos($url, 'ftp.') === 0)
            $full_url = 'ftp://' . $full_url;
        else if (!preg_match('#^([a-z0-9]{3,6})://#', $url, $bah))
            $full_url = 'http://' . $full_url;
        $link = ($link == '' || $link == $url) ? ((strlen($url) > 55) ? substr($url, 0, 39) . ' &hellip; ' . substr($url, -10) : $url) : stripslashes($link);
        $target = ($forum_user['new_window_link'] == '1') ? ' onclick="window.open(this.href); return false;"' : NULL;
        return '<a href="' . $full_url . '"' . $target . '>' . $link . '</a>';
    }

    function Handle_Img_Tag($url, $is_signature = false)
    {
        global $lang, $forum_user;
        $target = ($forum_user['new_window_link'] == '1') ? ' onclick="window.open(this.href); return false;"' : NULL;
        $img_tag = '<a href="' . $url . '"' . $target . '>&lt;' . $lang['Image link'] . '&gt;</a>';
        if ($is_signature && $forum_user['show_img_sig'] != '0')
            $img_tag = '<img class="sigimage" src="' . $url . '" alt="' . BB_Input::Htmlspecialchars($url) . '" />';
        else if (!$is_signature && $forum_user['show_img'] != '0')
            $img_tag = '<img class="postimg" src="' . $url . '" alt="' . BB_Input::Htmlspecialchars($url) . '" />';
        return $img_tag;
    }

    function Do_BBCode_Preview($text)
    {
        global $lang, $forum_user;
        if (strpos($text, 'quote') !== false)
        {
            $text = str_replace('[quote]', '', $text);
            $text = preg_replace('#\[quote=(&quot;|"|\'|)(.*)\\1\]#seU', '"".str_replace(array(\'[\', \'\\"\'), array(\'&#91;\', \'"\'), \'$2\')." ".$lang[\'wrote\'].""', $text);
            $text = preg_replace('#\[\/quote\]\s*#', '', $text);
        }
        $pattern = array('#\[b\](.*?)\[/b\]#s',
            '#\[i\](.*?)\[/i\]#s',
            '#\[u\](.*?)\[/u\]#s',
            '#\[url\]([^\[]*?)\[/url\]#e',
            '#\[url=([^\[]*?)\](.*?)\[/url\]#e',
            '#\[email\]([^\[]*?)\[/email\]#',
            '#\[email=([^\[]*?)\](.*?)\[/email\]#',
            '#\[color=([a-zA-Z]*|\#?[0-9a-fA-F]{6})](.*?)\[/color\]#s',
            '#\[font=(.*?)](.*?)\[/font\]#',
            '#\[align=(.*?)\](.*?)\[/align\]#',
            '#\[s\](.*?)\[/s\]#',
            '#\[pre\](.*?)\[/pre\]#',
            '#\[sup\](.*?)\[/sup\]#',
            '#\[hr]#',
            '#\[sub\](.*?)\[/sub\]#');
        $replace = array('$1',
            '$1',
            '$1',
            '',
            '',
            '',
            '',
            '$2',
            '$2',
            '$2',
            '$1',
            '$1',
            '$1',
            '',
            '$1');
        $text = preg_replace($pattern, $replace, $text);
        return $text;
    }

    function Do_BBCode($text)
    {
        global $lang, $forum_user, $cur_post;
        if (strpos($text, 'quote') !== false)
        {
            $text = str_replace('[quote]', '</p><blockquote><div class="incqbox"><p>', $text);
            $text = preg_replace('#\[quote=(&quot;|"|\'|)(.*)\\1\]#seU', '"</p><blockquote><div class=\"incqbox\"><h4>".str_replace(array(\'[\', \'\\"\'), array(\'&#91;\', \'"\'), \'$2\')." ".$lang[\'wrote\'].":</h4><p>"', $text);
            $text = preg_replace('#\[\/quote\]\s*#', '</p></div></blockquote><p>', $text);
        }
        if (strpos($text, 'list') !== false)
        {
            $text = str_replace('[list]', '</p><ul>', $text);
            $text = str_replace('[li]', '<li style="list-style: inside;">', $text);
            $text = str_replace('[/li]', '</li>', $text);
            $text = preg_replace('#\[\/list\]\s*#', '</ul><p>', $text);
        }
        if (strpos($text, 'spoiler') !== false)
        {
            $text = str_replace('[spoiler]', '</p><div class="spoilerbox"><h4>' . $lang['Spoiler'] . '</h4><p>', $text);
            $text = preg_replace('#\[\/spoiler\]\s*#', '</p></div><p>', $text);
        }
        if (strpos($text, 'youtube') !== false)
        {
            $text = str_replace("watch?v=", "v/", $text);
        }
        if (bb_message_bbcode_media == '1')
        {
            $pattern = array('#\[b\](.*?)\[/b\]#s',
                '#\[i\](.*?)\[/i\]#s',
                '#\[u\](.*?)\[/u\]#s',
                '#\[url\]([^\[]*?)\[/url\]#e',
                '#\[url=([^\[]*?)\](.*?)\[/url\]#e',
                '#\[email\]([^\[]*?)\[/email\]#e',
                '#\[email=([^\[]*?)\](.*?)\[/email\]#e',
                '#\[ul\](.*?)\[/ul\]#s',
                '#\[ol\](.*?)\[/ol\]#',
                '#\[uli\](.*?)\[/uli\]#',
                '#\[oli\](.*?)\[/oli\]#',
                '#\[color=([a-zA-Z]*|\#?[0-9a-fA-F]{6})](.*?)\[/color\]#s',
                '#\[font=(.*?)](.*?)\[/font\]#',
                '#\[align=(.*?)\](.*?)\[/align\]#',
                '#\[hr /\]#',
                '#\[hr\]#',
                '#\[s\](.*?)\[/s\]#',
                '#\[pre\](.*?)\[/pre\]#',
                '#\[sup\](.*?)\[/sup\]#',
                '#\[sub\](.*?)\[/sub\]#',
                '#\[size=([0-9]*)](.*?)\[/size\]#s',
                '#\[video\](.*?)\[/video\]#',
                '#\[flash\](.*?)\[/flash\]#',
                '#\[fvideo\](.*?)\[/fvideo\]#',
                '#\[realvideo\](.*?)\[/realvideo\]#',
                '#\[quicktime\](.*?)\[/quicktime\]#',
                '#\[gvideo\](.*?)\[/gvideo\]#',
                '#\[youtube\](.*?)\[/youtube\]#',
                '#\[mp3\](.*?)\[/mp3\]#',
                '#\[h\](.*?)\[/h\]#');
            $replace = array('<strong>$1</strong>',
                '<em>$1</em>',
                '<span class="bbu">$1</span>',
                'BB_Parser::Handle_Url_Tag(\'$1\')',
                'BB_Parser::Handle_Url_Tag(\'$1\', \'$2\')',
                'BB_Parser::Handle_Email_Tag(\'$1\')',
                'BB_Parser::Handle_Email_Tag(\'$1\',\'$2\')',
                '<ul>$1</ul>',
                '<ol>$1</ol>',
                '<li>&#149;&nbsp;&nbsp;$1</li>',
                '<li>$1</li>',
                '<span style="color: $1">$2</span>',
                '<span style="font-family: $1">$2</span>',
                '<div align="$1">$2</div>',
                '<hr />',
                '<hr />',
                '<del>$1</del>',
                '<pre>$1</pre>',
                '<sup>$1</sup>',
                '<sub>$1</sub>',
                '<span style="font-size: $1px">$2</span>',
                '<div class="xenno_video"><OBJECT ID="MediaPlayer" WIDTH="100%" HEIGHT="100%" CLASSID="CLSID:22D6F312-B0F6-11D0-94AB-0080C74C7E95" STANDBY="Loading Windows Media Player components..." TYPE="application/x-oleobject"><PARAM NAME="FileName" VALUE="$1"><PARAM name="autostart" VALUE="false"><PARAM name="ShowControls" VALUE="true"><param name="ShowStatusBar" value="false"><PARAM name="ShowDisplay" VALUE="false"><EMBED TYPE="application/x-mplayer2" SRC="$1" NAME="MediaPlayer" WIDTH="100%" HEIGHT="100%" ShowControls="1" ShowStatusBar="0" ShowDisplay="0" autostart="0"></EMBED></OBJECT></div>',
                '<div class="xenno_video"><object type="application/x-shockwave-flash" data="$1" width="100%" height="100%" wmode="transparent"><param name="movie" value="' . bb_forum_url . '/include/js/flvplayer.swf?file=$1&autoStart=false" /><param name="wmode" value="transparent" /></object></div>',
                '<div class="xenno_video"><object type="application/x-shockwave-flash" data="' . bb_forum_url . '/include/js/flvplayer.swf?file=$1&autoStart=false" width="100%" height="100%" wmode="transparent"><param name="movie" value="' . bb_forum_url . '/include/js/flvplayer.swf?file=$1&autoStart=false" /><param name="wmode" value="transparent" /></object></div>',
                '<div class="xenno_video"><embed src="$1" width="100%" height="100%" type="audio/x-pn-realaudio-plugin" console="Clip1" autostart="true" controls="ImageWindow"><br><embed src="$1" width="100%" height="20" type="audio/x-pn-realaudio-plugin" console="Clip1" autostart="true" controls="ControlPanel"></embed></div>',
                '<div class="xenno_video"><object classid="clsid:02BF25D5-8C17-4B23-BC80-D3488ABDDC6B" codebase="http://www.apple.com/qtactivex/qtplugin.cab#version=6,0,2,0" width="100%" height="100%"><param name="controller" value="false"><param name="type" value="video/quicktime"><param name="autoplay" value="false"><param name="target" value="myself"><param name="src" value="http://www.themoviebox.net/trailers/CLICK-IMAGE.gif"><param name="pluginspage" value="http://www.apple.com/quicktime/download/indext.html"><embed width="100%" height="100%" controller="false" target="myself" src="http://www.themoviebox.net/trailers/CLICK-IMAGE.gif" href="$1" type="video/quicktime" bgcolor="#f7f7f7" border="0" pluginspage="http://www.apple.com/quicktime/download/indext.html" autoplay="false"></embed></object></div>',
                '<div class="xenno_video"><object type="application/x-shockwave-flash" data="http://video.google.com/googleplayer.swf?docId=$1" width="100%" height="100%" wmode="transparent"><param name="movie" value="http://video.google.com/googleplayer.swf?docId=$1" /><param name="wmode" value="transparent" /></object></div>',
                '<div class="xenno_video"><object type="application/x-shockwave-flash" data="$1" width="100%" height="100%" wmode="transparent"><param name="movie" value="$1" /><param name="wmode" value="transparent" /></object></div>',
                '<div class="xenno_audio"><object type="application/x-shockwave-flash" data="' . bb_forum_url . '/include/js/mp3player.swf?mp3=$1&bgcolor=ffffff" width="240" height="20" wmode="transparent"><param name="movie" value="' . bb_forum_url . '/include/js/mp3player.swf?mp3=$1&bgcolor=ffffff" /><param name="wmode" value="transparent" /></object></div>',
                '<span style="background-color: #FFFF00; color: #000000">$1</span>');
        }
        else
        {
            $pattern = array('#\[b\](.*?)\[/b\]#s',
                '#\[i\](.*?)\[/i\]#s',
                '#\[u\](.*?)\[/u\]#s',
                '#\[url\]([^\[]*?)\[/url\]#e',
                '#\[url=([^\[]*?)\](.*?)\[/url\]#e',
                '#\[email\]([^\[]*?)\[/email\]#e',
                '#\[email=([^\[]*?)\](.*?)\[/email\]#e',
                '#\[ul\](.*?)\[/ul\]#s',
                '#\[ol\](.*?)\[/ol\]#',
                '#\[uli\](.*?)\[/uli\]#',
                '#\[oli\](.*?)\[/oli\]#',
                '#\[color=([a-zA-Z]*|\#?[0-9a-fA-F]{6})](.*?)\[/color\]#s',
                '#\[font=(.*?)](.*?)\[/font\]#',
                '#\[align=(.*?)\](.*?)\[/align\]#',
                '#\[hr /\]#',
                '#\[hr\]#',
                '#\[s\](.*?)\[/s\]#',
                '#\[pre\](.*?)\[/pre\]#',
                '#\[sup\](.*?)\[/sup\]#',
                '#\[sub\](.*?)\[/sub\]#',
                '#\[size=([0-9]*)](.*?)\[/size\]#s',
                '#\[h\](.*?)\[/h\]#');
            $replace = array('<strong>$1</strong>',
                '<em>$1</em>',
                '<span class="bbu">$1</span>',
                'BB_Parser::Handle_Url_Tag(\'$1\')',
                'BB_Parser::Handle_Url_Tag(\'$1\', \'$2\')',
                'BB_Parser::Handle_Email_Tag(\'$1\')',
                'BB_Parser::Handle_Email_Tag(\'$1\',\'$2\')',
                '<ul>$1</ul>',
                '<ol>$1</ol>',
                '<li>&#149;&nbsp;&nbsp;$1</li>',
                '<li>$1</li>',
                '<span style="color: $1">$2</span>',
                '<span style="font-family: $1">$2</span>',
                '<div align="$1">$2</div>',
                '<hr />',
                '<hr />',
                '<del>$1</del>',
                '<pre>$1</pre>',
                '<sup>$1</sup>',
                '<sub>$1</sub>',
                '<span style="font-size: $1px">$2</span>',
                '<span style="background-color: #FFFF00; color: #000000">$1</span>');
        }
        $text = preg_replace($pattern, $replace, $text);
        return $text;
    }

    function Do_Clickable($text)
    {
        global $forum_user;
        $text = ' ' . $text;
        $text = preg_replace('#([\s\(\)])(https?|ftp|news){1}://([\w\-]+\.([\w\-]+\.)*[\w]+(:[0-9]+)?(/[^"\s\(\)<\[]*)?)#ie', '\'$1\'.BB_Parser::Handle_Url_Tag(\'$2://$3\')', $text);
        $text = preg_replace('#([\s\(\)])(www|ftp)\.(([\w\-]+\.)*[\w]+(:[0-9]+)?(/[^"\s\(\)<\[]*)?)#ie', '\'$1\'.BB_Parser::Handle_Url_Tag(\'$2.$3\', \'$2.$3\')', $text);
        $text = preg_replace('#([\s\(\)])((mailto:)?([\w\d][\w\d$.-]*[\w\d]@[\w\d][\w\d.-]*[\w\d]\.[a-z0-9]{2,5}))((/[^"\s\(\)<\[]*)?)#ie', '\'$1\'.BB_Parser::Handle_Email_Tag(\'$2\')', $text);
        return substr($text, 1);
    }

    function Do_Smilies($text)
    {
        global $smiley_text, $smiley_img;
        $text = ' ' . $text . ' ';
        $num_smilies = count($smiley_text);
        for ($i = 0; $i < $num_smilies; ++$i)
            $text = preg_replace("#(?<=.\W|\W.|^\W)" . preg_quote($smiley_text[$i], '#') . "(?=.\W|\W.|\W$)#m", '$1<img src="' . bb_forum_url . 'img/smilies/' . $smiley_img[$i] . '" alt="' . substr($smiley_img[$i], 0, strrpos($smiley_img[$i], '.')) . '" />$2', $text);
        return substr($text, 1, -1);
    }

    function Parse_Code($codename, $code)
    {
        global $lang, $forum_user;
        if (!empty($codename))
            require_once(FORUM_ROOT . 'include/functions/SyntaxHighlight.php');
        else
        {
            if (bb_indent_num_spaces != 8)
            {
                $spaces = str_repeat(' ', bb_indent_num_spaces);
                $code = str_replace("\t", $spaces, trim($code));
            }
        }
        $num_lines = ((substr_count($code, "\n")) + 3) * 1.5;
        $height_str = ($num_lines > 35) ? '35em' : $num_lines . 'em';
        return '</p><div class="codebox"><div class="incqbox"><div class="scrollbox" style="height: ' . $height_str . '"><pre dir="ltr">' . $code . '</pre></div></div></div><p>';
    }

    function Parse_Part($text, $hide_smilies)
    {
        global $lang, $forum_user;
        if (bb_censoring == '1')
            $text = BB_Input::Censor_Words($text);
        $text = BB_Input::Htmlspecialchars($text);
        if (bb_make_links == '1')
            $text = BB_Parser::Do_Clickable($text);
        if (bb_smilies == '1' && $forum_user['show_smilies'] == '1' && $hide_smilies == '0')
            $text = BB_Parser::Do_Smilies($text);
        if (bb_message_bbcode == '1' && strpos($text, '[') !== false && strpos($text, ']') !== false)
        {
            $text = BB_Parser::Do_BBCode($text);
            if (bb_message_img_tag == '1')
                $text = preg_replace('#\[img\]((ht|f)tps?://)([^\s<"]*?)\[/img\]#e', 'BB_Parser::Handle_Img_Tag(\'$1$3\')', $text);
        }
        $pattern = array("\n", "\t", '  ', '  ');
        $replace = array('<br />', '&nbsp; &nbsp; ', '&nbsp; ', ' &nbsp;');
        $text = str_replace($pattern, $replace, $text);
        return $text;
    }

    function Parse_Part_Preview($text, $hide_smilies)
    {
        global $lang, $forum_user;
        if (bb_censoring == '1')
            $text = BB_Input::Censor_Words($text);
        if (bb_message_bbcode == '1' && strpos($text, '[') !== false && strpos($text, ']') !== false)
            $text = BB_Parser::Do_BBCode_Preview($text);
        $pattern = array("\n", "\t", '  ', '  ');
        $replace = array('<br />', '&nbsp; &nbsp; ', '&nbsp; ', ' &nbsp;');
        $text = str_replace($pattern, $replace, $text);
        return $text;
    }

    function Parse_Message_Preview($text, $hide_smilies = 1)
    {
        global $lang, $forum_user;
        $text = preg_replace('#\[noparse\](.*)\[/noparse\]#sUe', 'BB_Parser::No_Parse_Message(\'$1\')', $text);
        if (strstr($text, "[code]"))
        {
            $text = str_replace('<', '&lt;', $text);
            $text = str_replace('>', '&gt;', $text);
        }
        $parts = preg_split('#\n?\[\/?code=?(.*?)\]\n?#is', $text, -1, PREG_SPLIT_DELIM_CAPTURE);
        $idx = 0;
        $codename = '';
        $text = '';
        foreach ($parts as $value)
        {
            $idx++;
            switch ($idx)
            {
                case 1:
                    $text .= BB_Parser::Parse_Part_Preview($value, $hide_smilies);
                    break;
                case 2:
                    $codename = $value;
                    break;
                case 4:
                    $codename = $value;
                    $idx = 0;
                    break;
            }
        }
        $text = str_replace(array('<p></p>', "(", ")"), array('', "\(", "\)"), '<p>' . $text . '</p>');
        $text = str_replace('**-NoParseTagBegin-**', '[', $text);
        $text = str_replace('**-NoParseTagEnd-**', ']', $text);
        return $text;
    }

    function Parse_Message($text, $hide_smilies)
    {
        global $lang, $forum_user;
        $text = preg_replace('#\[noparse\](.*)\[/noparse\]#sUe', 'BB_Parser::No_Parse_Message(\'$1\')', $text);
        if (strstr($text, "[code]"))
        {
            $text = str_replace('<', '&lt;', $text);
            $text = str_replace('>', '&gt;', $text);
        }
        $parts = preg_split('#\n?\[\/?code=?(.*?)\]\n?#is', $text, -1, PREG_SPLIT_DELIM_CAPTURE);
        $idx = 0;
        $codename = '';
        $text = '';
        foreach ($parts as $value)
        {
            $idx++;
            switch ($idx)
            {
                case 1:
                    $text .= BB_Parser::Parse_Part($value, $hide_smilies);
                    break;
                case 2:
                    $codename = $value;
                    break;
                case 3:
                    $text .= BB_Parser::Parse_Code(strtolower($codename), $value);
                    break;
                case 4:
                    $codename = $value;
                    $idx = 0;
                    break;
            }
        }
        $text = str_replace('<p></p>', '', '<p>' . $text . '</p>');
        $text = str_replace('**-NoParseTagBegin-**', '[', $text);
        $text = str_replace('**-NoParseTagEnd-**', ']', $text);
        return $text;
    }

    function No_Parse_Message($text_noparse)
    {
        $text_noparse = str_replace('[', '**-NoParseTagBegin-**', $text_noparse);
        $text_noparse = str_replace(']', '**-NoParseTagEnd-**', $text_noparse);
        return $text_noparse;
    }

    function Parse_Signature($text)
    {
        global $lang, $forum_user;
        if (bb_censoring == '1')
            $text = BB_Input::Censor_Words($text);
        $text = BB_Input::Htmlspecialchars($text);
        if (bb_make_links == '1')
            $text = BB_Parser::Do_Clickable($text);
        if (bb_smilies_sig == '1' && $forum_user['show_smilies'] != '0')
            $text = BB_Parser::Do_Smilies($text);
        if (bb_sig_bbcode == '1' && strpos($text, '[') !== false && strpos($text, ']') !== false)
        {
            $text = BB_Parser::Do_BBCode($text);
            if (bb_sig_img_tag == '1')
                $text = preg_replace('#\[img\]((ht|f)tps?://)([^\s<"]*?)\[/img\]#e', 'BB_Parser::Handle_Img_Tag(\'$1$3\', true)', $text);
        }
        $pattern = array("\n", "\t", '  ', '  ');
        $replace = array('<br />', '&nbsp; &nbsp; ', '&nbsp; ', ' &nbsp;');
        $text = str_replace($pattern, $replace, $text);
        return $text;
    }

    function Handle_Email_Tag($email, $text = '')
    {
        $enc_email = '';
        for ($a = 0; $a < strlen($email); $a++)
        {
            $charValue = ord(substr($email, $a, 1));
            $charValue+=intval(2);
            $enc_email.=chr($charValue);
        }
        $enc_email = str_replace('\\', '\\\\', $enc_email);
        $enc_email = str_replace('\'', '\\\'', $enc_email);
        if ($text == '')
            $text = str_replace('@', '@<span style="display:none">remove-this.</span>', $email);
        return '<a href="javascript:xBB_Mail_To(\'' . $enc_email . '\')">' . $text . '</a>';
    }

}

?>
